@@ -13,6 +13,7 @@ SELECT2_CSS = getattr(settings, 'SELECT2_CSS',
 SELECT2_USE_BUNDLED_JQUERY = getattr(settings, 'SELECT2_USE_BUNDLED_JQUERY', True)
 
 SELECT2_WIDGET_JS = [
+    static('easy_select2/js/init.js'),
     static('easy_select2/js/easy_select2.js'),
     static('easy_select2/js/lookup_override.js'),
     static(SELECT2_JS),
@@ -29,48 +30,61 @@ class Select2Mixin(object):
 
     Generally should be mixed with widgets that render select input.
     """
-
-    inline_script = """
-        <script>
-            $("#%(id)s").on('select2changed', function(e){
-                $("#%(id)s").select2(%(options)s);
-            }).trigger('select2changed');
-        </script>
-    """
+    html = """<div class="field-easy-select2" style="display:none" id="{id}" {options}></div>"""
 
     def __init__(self, select2attrs=None, *args, **kwargs):
         """
-        Initializes default select2 attributes.
+        Initialize default select2 attributes.
 
         If width is not provided, sets Select2 width to 250px.
 
         Args:
-            select2attrs: a dictionary or string, which then
-                converted to unicode string representation and
-                passed to Select2 constructor function as options.
+            select2attrs: a dictionary, which then passed to
+                Select2 constructor function as options.
         """
         self.select2attrs = select2attrs or {}
+        assert_msg = "select2attrs attribute must dict not {}"
+        assert isinstance(self.select2attrs, dict), assert_msg.format(
+            self.select2attrs.__class__.__name__
+        )
         if not 'width' in self.select2attrs:
             self.select2attrs.update({'width': '250px'})
         super(Select2Mixin, self).__init__(*args, **kwargs)
 
+    # This function is taken from django-select2
+    def get_options(self):
+        """Return dictionary of options to be used by Select2."""
+        return self.select2attrs
+
+    # This function is taken from django-select2
+    def render_select2_options_code(self, options, id_):
+        """Render options for select2."""
+        output = []
+        for key, value in options.items():
+            if isinstance(value, (dict, list)):
+                value = json.dumps(value)
+            output.append("data-{name}='{value}'".format(
+                name=key,
+                value=mark_safe(value)))
+        return mark_safe(' '.join(output))
+
+    def render_js_code(self, id_, *args, **kwargs):
+        """Render html container for Select2 widget with options."""
+        if id_:
+            options = self.render_select2_options_code(
+                dict(self.get_options()), id_)
+            return mark_safe(self.html.format(id=id_, options=options))
+        return u''
+
     def render(self, *args, **kwargs):
         """
-        Extends base class's `render` method by appending
+        Extend base class's `render` method by appending
         javascript inline text to html output.
         """
         output = super(Select2Mixin, self).render(*args, **kwargs)
         id_ = kwargs['attrs']['id']
+        output += self.render_js_code(id_, *args, **kwargs)
 
-        if isinstance(self.select2attrs, str):
-            options = self.select2attrs
-        else:
-            options = json.dumps(self.select2attrs)
-
-        output += self.inline_script % {
-            'id': id_,
-            'options': options,
-        }
         return mark_safe(output)
 
     class Media:
@@ -83,12 +97,12 @@ class Select2Mixin(object):
 
 
 class Select2(Select2Mixin, forms.Select):
-    """Implements single-valued select widget with Select2."""
+    """Implement single-valued select widget with Select2."""
     pass
 
 
 class Select2Multiple(Select2Mixin, forms.SelectMultiple):
-    """Implements multiple select widget with Select2."""
+    """Implement multiple select widget with Select2."""
     pass
 
 
@@ -108,18 +122,6 @@ class Select2TextMixin(Select2Mixin):
 
     def __init__(self, select2attrs=None, *args, **kwargs):
         super(Select2TextMixin, self).__init__(select2attrs, *args, **kwargs)
-        if 'data' in self.select2attrs:
-            self.inline_script = """
-                <script>
-                    (function(){
-                        var options = %(options)s;
-                        options['createSearchChoice'] = function(term){
-                            return { 'id': term, 'text': term };
-                        };
-                        $("#%(id)s").select2(options);
-                    }());
-                </script>
-            """
 
 
 class Select2TextInput(Select2TextMixin, forms.TextInput):
